<?php

declare(strict_types=1);

namespace Drupal\presigned_url\Hook;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\presigned_url\PresignedUrlService;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * File related hooks.
 */
class FileHooks {

  public function __construct(
    protected readonly StreamWrapperManagerInterface $streamWrapperManager,
    protected readonly RequestStack $requestStack,
    protected readonly PresignedUrlService $presignedUrlService,
  ) {}

  /**
   * Handles file download access.
   */
  #[Hook('file_access')]
  public function handleFileDownloadAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    if ($operation === 'download') {
      $request = $this->requestStack->getCurrentRequest();

      // If the URL of the current request is valid, allow access to download.
      // The actual file access is handled by the `file_download` hook in
      // the `file` module.
      if ($this->presignedUrlService->validatePresignedUrl($request)) {
        return AccessResult::allowed();
      }
    }

    return AccessResult::neutral();
  }

}
